!-------------------------------------------------------------------------------

!VERS

! This file is part of Code_Saturne, a general-purpose CFD tool.
!
! Copyright (C) 1998-2014 EDF S.A.
!
! This program is free software; you can redistribute it and/or modify it under
! the terms of the GNU General Public License as published by the Free Software
! Foundation; either version 2 of the License, or (at your option) any later
! version.
!
! This program is distributed in the hope that it will be useful, but WITHOUT
! ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
! FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
! details.
!
! You should have received a copy of the GNU General Public License along with
! this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
! Street, Fifth Floor, Boston, MA 02110-1301, USA.

!-------------------------------------------------------------------------------


subroutine uslain &
!================

 ( nvar   , nscal  ,                                              &
   ntersl , nvlsta , nvisbr ,                                     &
   nptnew , iprev  ,                                              &
   itypfb , itrifb , ifrlag , injfac ,                            &
   dt     ,                                                       &
   icocel , lndnod , itycel , dlgeo,                              &
   ncmax  , nzmax  , iusloc )

!===============================================================================
! Purpose:
! --------
!
! User subroutine of the Lagrangian particle-tracking module:
! -----------------------------------------
!
! User subroutine (non-mandatory intervention)

! User subroutine for the boundary conditions for the particles
! (inlet and treatment for the other boundaries)
!
! This routine is called after the initialization of the
! eptp, pepa and ipepa arrays for the new particles in order to modify them
! to inject new particle profiles.
!

!-------------------------------------------------------------------------------
! Arguments
!__________________.____._____.________________________________________________.
! name             !type!mode ! role                                           !
!__________________!____!_____!________________________________________________!
! nvar             ! i  ! <-- ! total number of variables                      !
! nscal            ! i  ! <-- ! total number of scalars                        !
! ntersl           ! i  ! <-- ! number of source terms of return coupling      !
! nvlsta           ! i  ! <-- ! nb of Lagrangian statistical variables         !
! nvisbr           ! i  ! <-- ! number of boundary statistics                  !
! nptnew           ! i  ! <-- ! total number of new particles for all the      !
!                  !    !     ! injection zones                                !
! iprev            ! i  ! <-- ! time step indicator for fields                 !
!                  !    !     !   0: use fields at current time step           !
!                  !    !     !   1: use fields at previous time step          !
! itrifb(nfabor)   ! ia ! <-- ! indirection for the sorting of the boundary    !
! itypfb(nfabor)   ! ia ! <-- ! type of the boundary faces                     !
! ifrlag(nfabor)   ! ia ! --> ! type of the Lagrangian boundary faces          !
! injfac(npbmax)   ! ia ! <-- ! number of the injection boundary face          !
! dt(ncelet)       ! ra ! <-- ! time step (per cell)                           !
! icocel           ! ia ! <-- ! connectivity cells -> faces                    !
!   (lndnod)       !    !     !    boundary cell if the number is negative     !
! lndnod           ! i  ! <-- ! dim. connectivity cells -> faces               !
!  itycel          ! ia ! <-- ! connectivity cells -> faces                    !
! (ncelet+1)       !    !     !    pointer of the icocel array                 !
!  dlgeo           ! ra ! <-- ! array of the geometrical quantities            !
!(nfabor,ngeol)    !    !     ! related to the boundary faces                  !
! ncmax            ! i  ! <-- ! number of class                                !
! nzmax            ! i  ! <-- ! number of zones                                !
! iusloc           ! ia ! <-- ! local equivalent of the iuslag array           !
!__________________!____!_____!________________________________________________!

!     Type: i (integer), r (real), s (string), a (array), l (logical),
!           and composite types (ex: ra real array)
!     mode: <-- input, --> output, <-> modifies data, --- work array

!===============================================================================

!===============================================================================
! Module files
!===============================================================================

use paramx
use numvar
use optcal
use cstnum
use cstphy
use entsor
use lagpar
use lagran
use ppppar
use ppthch
use cpincl
use mesh

!===============================================================================

implicit none

! Arguments

integer          nvar   , nscal
integer          ntersl , nvlsta , nvisbr
integer          nptnew
integer          iprev
integer          lndnod

integer          itypfb(nfabor) , itrifb(nfabor)
integer          ifrlag(nfabor)
integer          injfac(nbpart+nptnew)

double precision dt(ncelet)
integer          icocel(lndnod) ,  itycel(ncelet+1)
double precision dlgeo(nfabor,ngeol)

integer ncmax, nzmax
integer iusloc(ncmax, nzmax, ndlaim)

! Local variables

integer          iclas , izone , ifac
integer          ii , ip , npt , npar1 , npar2, ipnorm

! User-defined local variables
! (the dimension of vgauss is 3, but 2 would be sufficient here)

double precision vgauss(3)

!===============================================================================

if (nbpnew.eq.0) return

!===============================================================================
! 1. Memory management
!===============================================================================

!===============================================================================
! 2. Initializations
!===============================================================================

!===============================================================================
! 3. Modification of properties of the new particles (injection profiles,
!    position of the injection point, statistical
!    weights, correction of the diameter if the standard-deviation option
!    is activated.)
!===============================================================================

!    These modifications occur after all the initializations related to
!    the particle injection, but before the treatment of the continuous
!    injection: it is thus possible to impose an injection profile with
!    the continous-injection option.
!

! reinitialization of the counter of the new particles
npt = nbpart

! for each boundary zone:
do ii = 1,nfrlag
  izone = ilflag(ii)

  ! for each class:
  do iclas = 1, iusncl(izone)

    ! if new particles must enter the domain:
    if (mod(ntcabs,iusloc(iclas,izone,ijfre)).eq.0) then

      do ip = npt+1 , npt+iusloc(iclas,izone,ijnbp)

        ! number of the original boundary face of injection

        ifac = injfac(ip)

        call inlet2(eptp(jup,ip), eptp(jvp,ip), eptp(jwp,ip), eptp(jzp,ip))

      enddo

      npt = npt + iusloc(iclas,izone,ijnbp)

    endif

  enddo
enddo

!
! Trick to average the statistics at iteration nstist
! starting from an unsteady two-coupling calculation
!

if (iplas.gt.nstist) then
   nstits = nstist
   isttio = 1
endif

!===============================================================================

!--------
! Formats
!--------

!----
! End
!----

return

end subroutine uslain

!===============================================================================


subroutine uslast &
!================

 ( nvar   , nscal  ,                                              &
   ntersl , nvlsta , nvisbr ,                                     &
   dt     )

!===============================================================================
! Purpose:
! --------
!
! User subroutine of the Lagrangian particle-tracking module:
! -----------------------------------------
!
! User subroutine (non-mandatory intervention)

! User-defined modifications on the variables at the end of the
! Lagrangian iteration and calculation of user-defined
! additional statistics on the particles.
!
! About the user-defined additional statistics, we recall that:
!

!   isttio = 0 : unsteady Lagrangian calculation
!          = 1 : steady Lagrangian calculation

!   istala : calculation of the statistics if >= 1, else no stats

!   isuist : Restart of statistics calculation if >= 1, else no stats

!   idstnt : Number of the time step for the start of the statistics calculation

!   nstist : Number of the Lagrangian iteration of the start of the steady computation

!   npst   : Number of iterations of the computation of the steady statistics

!   npstt  : Total number of iterations of the statistics calculation since the
!            beginning of the calculation, including the unsteady part

!   tstat  : Physical time of the recording of the steady volume statistics
!            (for the unsteady part, tstat = dtp the Lagrangian time step)
!

!-------------------------------------------------------------------------------
! Arguments
!__________________.____._____.________________________________________________.
! name             !type!mode ! role                                           !
!__________________!____!_____!________________________________________________!
! nvar             ! i  ! <-- ! total number of variables                      !
! nscal            ! i  ! <-- ! total number of scalars                        !
! ntersl           ! i  ! <-- ! number of source terms of return coupling      !
! nvlsta           ! i  ! <-- ! nb of Lagrangian statistical variables         !
! nvisbr           ! i  ! <-- ! number of boundary statistics                  !
! dt(ncelet)       ! ra ! <-- ! time step (per cell)                           !
!__________________!____!_____!________________________________________________!

!     Type: i (integer), r (real), s (string), a (array), l (logical),
!           and composite types (ex: ra real array)
!     mode: <-- input, --> output, <-> modifies data, --- work array
!===============================================================================

!===============================================================================
! Module files
!===============================================================================

use paramx
use numvar
use cstnum
use optcal
use pointe
use entsor
use lagpar
use lagran
use cstphy
use parall
use ppppar
use ppthch
use cpincl
use mesh

!===============================================================================

implicit none

! Arguments

integer          nvar   , nscal
integer          ntersl , nvlsta , nvisbr

double precision dt(ncelet)

! Local variables

integer          npt ,  iel

integer          ivf , ivff , iflu , icla

! User-defined local variables

integer          nxlist
parameter       (nxlist=100)

integer          iplan
integer          ii, ind, il, jj
integer          inoeud, irang0, indic
integer          ist(6)

double precision zz(4), zzz(8), xlist(nxlist,8), xyzpt(3)

double precision, allocatable, dimension(:) :: tabvr

character        name(8)*4

double precision debm(4)
save             debm

double precision xi, xmoy, vx, vy, vx2, vz2, fv
integer          ifac , itabvr
integer          nc1, nc2, itri, itri2

integer ind_g

double precision, allocatable, dimension(:,:) :: xlist_g
double precision, allocatable, dimension(:) :: sortaux

!===============================================================================

!===============================================================================
! 4 - Extraction of volume statistics at the end of the calculation
!===============================================================================

if (ntcabs.eq.ntmabs) then

   zzz(1) = 0.005d0
   zzz(2) = 0.025d0
   zzz(3) = 0.050d0
   zzz(4) = 0.075d0
   zzz(5) = 0.100d0
   zzz(6) = 0.150d0
   zzz(7) = 0.200d0
   zzz(8) = 0.250d0

   name(1) = 'xl01'
   name(2) = 'xl05'
   name(3) = 'xl10'
   name(4) = 'xl15'
   name(5) = 'xl20'
   name(6) = 'xl30'
   name(7) = 'xl40'
   name(8) = 'xl50'

   ist(1) = ilvx
   ist(2) = ilvz
   ist(3) = ilvx
   ist(4) = ilvz
   ist(5) = ilfv
   ist(6) = ilpd

   npts = nxlist

   ! Allocate work arrays

   allocate(tabvr(ncelet))

   do iplan = 1,8

      if (irangp.le.0) then
        open(file=name(iplan), unit=impusr(1), form="formatted")
      endif

      do ivf = 1,6

         ivff = ist(ivf)
         icla = 0
         if ((ivf .eq. 3) .or. (ivf .eq. 4)) then
            iflu = 1
         else
            iflu = 0
         endif

         call uslaen                                               &
        !==========
        ( nvlsta ,                                                       &
          ivff   , ivff   , ivff   , iflu   , ilpd   , icla   ,          &
          tabvr  )

         ind = 0

         do ifac = 1,nfac

            if (cdgfac(1,ifac).lt.(zzz(iplan) + 1.d-5) .and.         &
                 cdgfac(1,ifac).ge.(zzz(iplan) - 1.d-5)) then

               ind = ind+1
               nc1 = ifacel(1,ifac)
               nc2 = ifacel(2,ifac)

               xlist(ind,1) = cdgfac(1,ifac)
               xlist(ind,2) = cdgfac(3,ifac) * (1.d3 / 5.d0)
               xlist(ind,ivf+2) = tabvr(nc1)*pond(ifac) +            &
                    tabvr(nc2)*(1.d0-pond(ifac))

            endif

         enddo

      enddo

      ind_g = ind
      if (irangp.ge.0) then
         call parcpt(ind_g)
      endif

      allocate(sortaux(ind_g))
      allocate(xlist_g(ind_g,8))

      if (irangp.ge.0) then
         do ii = 1, 8
            call paragv(ind, ind_g, xlist(:,ii), xlist_g(:,ii))
         enddo
      else
         do ii = 1, 8
            do jj = 1, ind_g
               xlist_g(jj,ii) = xlist(jj,ii)
            enddo
         enddo
      endif

      if (irangp.le.0) then

         do ii = 1, 6
            sortaux = xlist_g(:,2)
            call sortc2(sortaux, xlist_g(:, ii + 2), ind_g)
         enddo

         call sortc2(xlist_g(:,2), xlist_g(:,2), ind_g)

         do il = 1, ind_g

            xlist_g(il,5) = sqrt(max(0.d0, xlist_g(il,5)))
            xlist_g(il,6) = sqrt(max(0.d0,  xlist_g(il,6)))

            write (impusr(1),'(8e13.5)') (xlist_g(il,ii), ii=1,8)

         enddo

         close(impusr(1))

      endif

      deallocate(sortaux)
      deallocate(xlist_g)

   enddo

   deallocate(tabvr)

endif

!===============================================================================

!====
! End
!====

return

end subroutine uslast

subroutine sortc2(tab1,tab2,n)

implicit none
integer n
double precision tab1(n), tab2(n), tab3(n)
double precision tab4(n), tab5(n)

integer ns, ii, jj, kk
double precision tabmin, xx, yy, zz, ww

ns = 1
do ii = 1, n-1
  tabmin = 1.d20
  kk = 0
  do jj = ns,n
    if (tabmin.gt.tab1(jj)) then
      tabmin = tab1(jj)
      kk = jj
    endif
  enddo
  xx = tab2(kk)
  tab1(kk) = tab1(ns)
  tab2(kk) = tab2(ns)
  tab1(ns) = tabmin
  tab2(ns) = xx
  ns = ns + 1
enddo

return
end subroutine sortc2
