subroutine inlet2                               &
!================

 ( u21, v21, w21, z )



!===============================================================================
! Purpose:
! ----------

! Subroutine inlet2
! -----------------
!
!   User-defined subroutine for the validation test-case
!   "Particle-laden wall jet" (JET_2_PHI) for the validation
!   of Code_Saturne
!
!   Initialization of the inlet conditions of the particle velocity
!


!-------------------------------------------------------------------------------
! Arguments
!__________________.____._____.________________________________________________.
!    name          !type!mode !                   role                         !
!__________________!____!_____!________________________________________________!
!  u21             ! r  ! --> ! vertical component of the part. velocity       !
!  v21             ! r  ! --> ! tangential component of the part. velocity (=0)!
!  w21             ! r  ! --> ! transverse component of the part. velocity     !
!  z               ! r  ! <-- ! transverse coordinate                          !
!__________________!____!_____!________________________________________________!
!     Type: i (integer), r (real), s (string), a (array), l (logical),
!           and composite types (ex: ra real array)
!     mode: <-- input, --> output, <-> modifies data, --- work array
!===============================================================================


!===============================================================================
!     Module files
!===============================================================================

use paramx
use entsor

!===============================================================================
implicit none

! Arguments

double precision u21, v21, w21, z

! Local variables

double precision zi(1:10), ui(1:10), wi(1:10)
double precision uf(1:10), wf(1:10), lvf(1:10)
double precision uvi(1:10)
double precision vgauss(3), teta, uvx, vxn, vzn, vpxn, vpzn, py
double precision up, wp, upp, wpp

integer itrouv, itmx, it, i, ip

!===============================================================================

!===============================================================================
! 1.  Data initializations with experimental measurements
!===============================================================================

!     Transverse coordinate

data zi/ 0.e-3, 1.e-3, 1.5e-3, 2.0e-3, 2.5e-3, 3.0e-3, 3.5e-3,    &
         4.0e-3, 4.5e-3, 5.0e-3 /

!      Vertical mean velocity of the particles

data ui/ 5.544, 8.827, 9.068, 9.169, 8.923, 8.295, 7.151, 6.048,  &
         4.785, 5.544 /

!      Transverse mean velocity of the particles

data wi/ 0. , 0.179, 0.206, 0.221, 0.220, 0.223, 0.206, 0.190,    &
         0.195, 0.504 /

!     Fluctuation of the vertical velocity of the particles

data uf/ 0.352, 0.352, 0.275, 0.252, 0.367, 0.516, 0.657, 0.872,  &
         1.080, 0.792 /

!     Fluctuation of the transverse velocity of the particles


data wf/ 0.058, 0.058, 0.056, 0.056, 0.060, 0.063, 0.058, 0.072,  &
         0.091, 0.232 /

!   Shear-stress (currently not used) of the particle velocity

data uvi/ 0.0017, 0.0017,  0.0016,  0.0027,  0.0077,              &
          0.0146, 0.0206,  0.0447,  0.0752,  0.1145 /

!===============================================================================
! 2.  Initialization of the constants
!===============================================================================

itmx= 9

!===============================================================================
! 3.  Interpolation
!===============================================================================

itrouv = 0
if ( z.lt.zi(itmx) .and. z.gt.zi(1))then
  do i=1,8
    if ( z.ge.zi(i) .and. z.lt.zi(i+1) ) then
      if ( itrouv.eq.1 ) then
        write(nfecra,9001)
        call csexit(1)
      else
        it = i
        itrouv = 1
      endif
    endif
  enddo
else if ( z.ge.zi(itmx) ) then
  it = itmx
else if ( z.le.zi(1) )then
  it = 1
else
  write(nfecra,9002)
  call csexit(1)
endif

!===============================================================================
! 4.  Calculation of particles velocity
!===============================================================================

up  = ui(it)  +(z-zi(it))*(ui(it+1)-ui(it))/(zi(it+1)-zi(it))

! The value of the mean transverse velocity is currently set to zero
! due to uncertainties on this variable
!
if (.false.) then
   wp  = wi(it)  +(z-zi(it))*(wi(it+1)-wi(it))/(zi(it+1)-zi(it))
else
   wp = 0.d0
endif


upp = uf(it)  +(z-zi(it))*(uf(it+1)-uf(it))/(zi(it+1)-zi(it))
wpp = wf(it)  +(z-zi(it))*(wf(it+1)-wf(it))/(zi(it+1)-zi(it))

! Calculations of the instantaneous particle velocity

call normalen(2,vgauss)

u21 = up  + vgauss(1) * upp
v21 = 0.d0
w21 = wp + vgauss(2) * wpp

!===============================================================================

!--------
! Formats
!--------

 9001 format(                                                           &
'@                                                            ',/,&
'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
'@                                                            ',/,&
'@ @@ CAUTION: STOP OF THE VALIDATION TEST-CASE               ',/,&
'@    =========   (INLET2)                                    ',/,&
'@                                                            ',/,&
'@    PROBLEM IN THE BOUNDARY CONDITIONS                      ',/,&
'@                                                            ',/,&
'@  The given coordinate is too large                         ',/,&
'@                                                            ',/,&
'@  The calculation cannot be run.                            ',/,&
'@                                                            ',/,&
'@  Please check the "inlet2" user-defined subroutine         ',/,&
'@                                                            ',/,&
'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
'@                                                            ',/)

 9002 format(                                                           &
'@                                                            ',/,&
'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
'@                                                            ',/,&
'@ @@ CAUTION: STOP OF THE VALIDATION TEST-CASE               ',/,&
'@    =========   (INLET2)                                    ',/,&
'@                                                            ',/,&
'@    PROBLEM IN THE BOUNDARY CONDITIONS                      ',/,&
'@                                                            ',/,&
'@  The interpolation has failed                              ',/,&
'@                                                            ',/,&
'@  The calculation cannot be run.                            ',/,&
'@                                                            ',/,&
'@  Please check the "inlet2" user-defined subroutine         ',/,&
'@                                                            ',/,&
'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
'@                                                            ',/)

!----
! End
!----

return

end subroutine inlet2
