subroutine inletp &
!================

 ( np, z, surf, poid, dtp, dp)


!==============================================================================
! Purpose:
! ----------

! Subroutine inletp
! -----------------
!
!   User-defined subroutine for the validation test-case
!   "Particle-laden wall jet" (JET_2_PHI) for the validation
!   of Code_Saturne
!
!   Initialization of the inlet conditions for the particle
!   volume fraction


!-------------------------------------------------------------------------------
! Arguments
!__________________.____._____.________________________________________________.
!    name          !type!mode !                   role                         !
!__________________!____!_____!________________________________________________!
!  np              ! r  ! --> ! number of particles to be injected             !
!  z               ! r  ! <-- ! transverse coordinate                          !
!  surf            ! r  ! <-- ! area of the particle injection zone            !
!  poids           ! r  ! <-- ! statistical weight of the injected particles   !
!  dtp             ! r  ! <-- ! time step of the Lagrangian iteration          !
!  dp              ! r  ! <-- ! diameter of the injected particles             !
!__________________!____!_____!________________________________________________!
!     Type: i (integer), r (real), s (string), a (array), l (logical),
!           and composite types (ex: ra real array)
!     mode: <-- input, --> output, <-> modifies data, --- work array
!===============================================================================


!===============================================================================
!      Module files
!===============================================================================

use paramx
use entsor
use cstnum
use cstphy

!===============================================================================
implicit none
! Arguments

double precision z, surf, poid, dtp

integer np

! Local variables

double precision zi(1:10), lvf(1:10), ui(1:10)
double precision part, rop, dp, py, vp, lvfp,up

integer itmx, it, i, itrouv

!===============================================================================

!===============================================================================
! 1.  Data initializations with experimental measurements
!===============================================================================

!     Transverse coordinate

data zi/ 0.e-3, 1.e-3, 1.5e-3, 2.0e-3, 2.5e-3, 3.0e-3, 3.5e-3,    &
         4.0e-3, 4.5e-3, 5.0e-3 /

!     Particle volume fraction

data lvf/ 0.377e-4, 2.236e-4, 3.014e-4, 4.306e-4, 5.689e-4,       &
          8.567e-4, 7.099e-4, 4.520e-4, 2.184e-4, 0.377e-4 /

!     Vertical mean velocity of the particles

data ui/ 5.544, 8.827, 9.068, 9.169, 8.923, 8.295, 7.151, 6.048,  &
         4.785, 5.544 /

!===============================================================================
! 2.  Initialization of the constants
!===============================================================================

itmx = 9
rop  = 2590.d0

!===============================================================================
! 3.  Interpolation
!===============================================================================



itrouv = 0
if ( z.lt.zi(itmx) .and. z.gt.zi(1))then
  do i = 1,8
    if ( z.ge.zi(i) .and. z.lt.zi(i+1) ) then
      if ( itrouv.eq.1 ) then
        write(nfecra,9001)
        call csexit(1)
      else
        it = i
        itrouv = 0
      endif
    endif
  enddo
else if ( z.ge.zi(itmx) ) then
  it = itmx
else if ( z.le.zi(1) )then
  it = 1
else
  write(nfecra,9002)
  call csexit(1)
endif

!===============================================================================
! 4.  Calculation of the volume fraction and of the statistical weight
!===============================================================================

up = ui(it) +(z-zi(it))*(ui(it+1)-ui(it))/(zi(it+1)-zi(it))
lvfp = lvf(it) +(z-zi(it))*(lvf(it+1)-lvf(it))/(zi(it+1)-zi(it))

! Calculation of the number of particles in the cell

vp = pi *(dp**3)/6.d0

part = lvfp * ( surf * up*dtp ) / vp


np = int(part)


if ( np .lt. 1 ) then
   np = 10
   poid = part / dble(np)

else if ( np .lt. 10 ) then
   np = np * 10
   poid = part / dble(np)

else
   np = int(part)
   poid = part / dble(np)

endif

 if (poid.lt.0.d0) then
   write(nfecra,9003)
   call csexit(1)
 endif

!===============================================================================

!--------
! Formats
!--------

 9001 format(                                                           &
'@                                                            ',/,&
'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
'@                                                            ',/,&
'@ @@ CAUTION: STOP OF THE VALIDATION TEST-CASE               ',/,&
'@    =========   (INLETP)                                    ',/,&
'@                                                            ',/,&
'@    PROBLEM IN THE BOUNDARY CONDITIONS                      ',/,&
'@                                                            ',/,&
'@  The given coordinate is too large                         ',/,&
'@                                                            ',/,&
'@  The calculation cannot be run.                            ',/,&
'@                                                            ',/,&
'@  Please check the "inletp" user-defined subroutine         ',/,&
'@                                                            ',/,&
'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
'@                                                            ',/)

 9002 format(                                                           &
'@                                                            ',/,&
'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
'@                                                            ',/,&
'@ @@ CAUTION: STOP OF THE VALIDATION TEST-CASE               ',/,&
'@    =========   (INLETP)                                    ',/,&
'@                                                            ',/,&
'@    PROBLEM IN THE BOUNDARY CONDITIONS                      ',/,&
'@                                                            ',/,&
'@  The interpolation has failed                              ',/,&
'@                                                            ',/,&
'@  The calculation cannot be run.                            ',/,&
'@                                                            ',/,&
'@  Please check the "inletp" user-defined subroutine         ',/,&
'@                                                            ',/,&
'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
'@                                                            ',/)

 9003 format(                                                           &
'@                                                            ',/,&
'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
'@                                                            ',/,&
'@ @@ CAUTION: STOP OF THE VALIDATION TEST-CASE               ',/,&
'@    =========   (INLETP)                                    ',/,&
'@                                                            ',/,&
'@    PROBLEM IN THE BOUNDARY CONDITIONS                      ',/,&
'@                                                            ',/,&
'@  The calculated statistical weight is negative             ',/,&
'@                                                            ',/,&
'@  The calculation cannot be run.                            ',/,&
'@                                                            ',/,&
'@  Please check the "inletp" user-defined subroutine         ',/,&
'@                                                            ',/,&
'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
'@                                                            ',/)



!----
! End
!----

return

end subroutine inletp
