!-------------------------------------------------------------------------------

!VERS

! This file is part of Code_Saturne, a general-purpose CFD tool.
!
! Copyright (C) 1998-2014 EDF S.A.
!
! This program is free software; you can redistribute it and/or modify it under
! the terms of the GNU General Public License as published by the Free Software
! Foundation; either version 2 of the License, or (at your option) any later
! version.
!
! This program is distributed in the hope that it will be useful, but WITHOUT
! ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
! FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
! details.
!
! You should have received a copy of the GNU General Public License along with
! this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
! Street, Fifth Floor, Boston, MA 02110-1301, USA.

!-------------------------------------------------------------------------------

subroutine uslag2 &
!================

 ( nvar   , nscal  ,                                              &
   ntersl , nvlsta , nvisbr ,                                     &
   itypfb , itrifb , ifrlag ,                                     &
   dt     )

!===============================================================================
! Purpose:
! ----------

!   Subroutine of the Lagrangian particle-tracking module:
!   -------------------------------------

!    User subroutine (Mandatory intervention)

!    User subroutine for the boundary conditions associated to the particles
!    (inlet and treatment of the other boundaries)


! Boundary faces identification
! =============================

! Boundary faces may be identified using the 'getfbr' subroutine.
! The syntax of this subroutine is described in the
! 'cs_user_boundary_conditions' subroutine,
! but a more thorough description can be found in the user guide.


!-------------------------------------------------------------------------------
! Arguments
!__________________.____._____.________________________________________________.
! name             !type!mode ! role                                           !
!__________________!____!_____!________________________________________________!
! nvar             ! i  ! <-- ! total number of variables                      !
! nscal            ! i  ! <-- ! total number of scalars                        !
! ntersl           ! i  ! <-- ! number of source terms of return coupling      !
! nvlsta           ! i  ! <-- ! nb of Lagrangian statistical variables         !
! nvisbr           ! i  ! <-- ! number of boundary statistics                  !
! itrifb(nfabor)   ! ia ! <-- ! indirection for the sorting of the             !
! itypfb(nfabor)   ! ia ! <-- ! type of the boundary faces                     !
! ifrlag(nfabor    ! ia ! --> ! type of the Lagrangian boundary faces          !
! dt(ncelet)       ! ra ! <-- ! time step (per cell)                           !
!__________________!____!_____!________________________________________________!

!     Type: i (integer), r (real), s (string), a (array), l (logical),
!           and composite types (ex: ra real array)
!     mode: <-- input, --> output, <-> modifies data, --- work array
!===============================================================================

!===============================================================================
! Module files
!===============================================================================

use parall
use paramx
use numvar
use optcal
use cstnum
use cstphy
use entsor
use lagpar
use lagran
use ppppar
use ppthch
use cpincl
use ihmpre
use mesh

!===============================================================================

implicit none

! Arguments

integer          nvar   , nscal
integer          ntersl , nvlsta , nvisbr

integer          itypfb(nfabor) , itrifb(nfabor)
integer          ifrlag(nfabor)

double precision dt(ncelet)

! Local variables

integer          ifac , izone, nbclas, iclas
integer          icha , n_zone_inj
integer          ilelt, nlelt

double precision pis6 , mp0 , temp

integer, dimension(ndlaim) :: iczpar
double precision, dimension(ndlagm) :: rczpar

integer, allocatable, dimension(:) :: lstelt

! Validation test-case
integer          iiii , iclg , icla
integer          i, n1, np, ii ,jj
double precision carpart(2,20), poid, dp, rd(1)

!===============================================================================


!===============================================================================
! 1.  Memory management
!===============================================================================

! Allocate a temporary array for boundary faces selection
allocate(lstelt(nfabor))


!===============================================================================
! 2. Initialization
!===============================================================================

pis6 = pi / 6.d0

do ii = 1,2
   do jj = 1, 20
      carpart(ii,jj) = 0.d0
   enddo
enddo

!===============================================================================
! 3. Construction of the boundary zones
!===============================================================================

! Definition of the boundary zones
! --------------------------------

! For the Lagrangian module, the user defines nfrlag boundary zones
! from the properties (groups, ...) of the boundary faces, from the
! boundary conditions, or even from their coordinates. To do this,
! we fill the ifrlag(nfabor) array which gives for every boundary
! face the number of the zone to which it belongs ifrlag(ifac)
!
! Be careful, all the boundary faces must have been assigned.
!
! The number of the zones (thus the values of ifrlag(ifac)) is arbitrarily
! chosen by the user, but must be a positive integer and inferior or equal
! to nflagm (parameter prescribed in lagpar.f90).
!
! Afterwards, we assign to every zone a type named itylag that will be used
! to prescribe global boundary conditions.

izone = -1

do ii = 1,2
   do jj = 1,20
      carpart(ii,jj) = 0.d0
   enddo
enddo

! Zone numbered 1 to 9 are particle injection zones

! ---> First zone
call getfbr('21 and z >= 0 and z <= 1.0e-3',nlelt,lstelt)
!==========

do ilelt = 1, nlelt

  ifac = lstelt(ilelt)

  izone        = 1
  ifrlag(ifac) = izone
  carpart(1,izone) = cdgfbo(3,ifac)
  carpart(2,izone) = sqrt( surfbo(1,ifac)*surfbo(1,ifac)       &
                          +surfbo(2,ifac)*surfbo(2,ifac)       &
                          +surfbo(3,ifac)*surfbo(3,ifac) )

enddo

! ---> Zone 2
call getfbr('21 and z > 1.0e-3 and z <= 1.5e-3',nlelt,lstelt)
!==========

do ilelt = 1, nlelt

  ifac = lstelt(ilelt)

  izone        = 2
  ifrlag(ifac) = izone
  carpart(1,izone) = cdgfbo(3,ifac)
  carpart(2,izone) = sqrt( surfbo(1,ifac)*surfbo(1,ifac)       &
                          +surfbo(2,ifac)*surfbo(2,ifac)       &
                          +surfbo(3,ifac)*surfbo(3,ifac) )
enddo

! ---> Zone 3
call getfbr('21 and Z > 1.5e-3 and Z <= 2.0e-3',nlelt,lstelt)
!==========

do ilelt = 1, nlelt

  ifac = lstelt(ilelt)

  izone        = 3
  ifrlag(ifac) = izone
  carpart(1,izone) = cdgfbo(3,ifac)
  carpart(2,izone) = sqrt( surfbo(1,ifac)*surfbo(1,ifac)       &
                          +surfbo(2,ifac)*surfbo(2,ifac)       &
                          +surfbo(3,ifac)*surfbo(3,ifac) )
enddo

! ---> Zone 4
call getfbr('21 and Z > 2.0e-3 and Z <= 2.5e-3',nlelt,lstelt)
!==========

do ilelt = 1, nlelt

  ifac = lstelt(ilelt)

  izone        = 4
  ifrlag(ifac) = izone
  carpart(1,izone) = cdgfbo(3,ifac)
  carpart(2,izone) = sqrt( surfbo(1,ifac)*surfbo(1,ifac)       &
                          +surfbo(2,ifac)*surfbo(2,ifac)       &
                          +surfbo(3,ifac)*surfbo(3,ifac) )
enddo

! ---> Zone 5
call getfbr('21 and Z > 2.5e-3 and Z <= 3.0e-3',nlelt,lstelt)
!==========

do ilelt = 1, nlelt

  ifac = lstelt(ilelt)

  izone        = 5
  ifrlag(ifac) = izone
  carpart(1,izone) = cdgfbo(3,ifac)
  carpart(2,izone) = sqrt( surfbo(1,ifac)*surfbo(1,ifac)       &
                          +surfbo(2,ifac)*surfbo(2,ifac)       &
                          +surfbo(3,ifac)*surfbo(3,ifac) )
enddo

! ---> Zone 6
call getfbr('21 and Z > 3.0e-3 and Z <= 3.5e-3',nlelt,lstelt)
!==========

do ilelt = 1, nlelt

  ifac = lstelt(ilelt)

  izone        = 6
  ifrlag(ifac) = izone
  carpart(1,izone) = cdgfbo(3,ifac)
  carpart(2,izone) = sqrt( surfbo(1,ifac)*surfbo(1,ifac)       &
                          +surfbo(2,ifac)*surfbo(2,ifac)       &
                          +surfbo(3,ifac)*surfbo(3,ifac) )
enddo

! ---> Zone 7
call getfbr('21 and Z > 3.5e-3 and Z <= 4.0e-3',nlelt,lstelt)
!==========

do ilelt = 1, nlelt

  ifac = lstelt(ilelt)

  izone        = 7
  ifrlag(ifac) = izone
  carpart(1,izone) = cdgfbo(3,ifac)
  carpart(2,izone) = sqrt( surfbo(1,ifac)*surfbo(1,ifac)       &
                          +surfbo(2,ifac)*surfbo(2,ifac)       &
                          +surfbo(3,ifac)*surfbo(3,ifac) )

enddo

! ---> Zone 8
call getfbr('21 and Z > 4.0e-3 and Z <= 4.5e-3',nlelt,lstelt)
!==========

do ilelt = 1, nlelt

  ifac = lstelt(ilelt)

  izone        = 8
  ifrlag(ifac) = izone
  carpart(1,izone) = cdgfbo(3,ifac)
  carpart(2,izone) = sqrt( surfbo(1,ifac)*surfbo(1,ifac)       &
                          +surfbo(2,ifac)*surfbo(2,ifac)       &
                          +surfbo(3,ifac)*surfbo(3,ifac) )
enddo

! ---> Zone 9
call getfbr('21 and Z > 4.5e-3 and Z <= 5.0e-3',nlelt,lstelt)
!==========

do ilelt = 1, nlelt

  ifac = lstelt(ilelt)

  izone        = 9
  ifrlag(ifac) = izone
  carpart(1,izone) = cdgfbo(3,ifac)
  carpart(2,izone) = sqrt( surfbo(1,ifac)*surfbo(1,ifac)       &
                          +surfbo(2,ifac)*surfbo(2,ifac)       &
                          +surfbo(3,ifac)*surfbo(3,ifac) )
enddo

! ---> Zone 10
call getfbr('4',nlelt,lstelt)
!==========

do ilelt = 1, nlelt

  ifac = lstelt(ilelt)

  izone        = 10
  ifrlag(ifac) = izone

enddo

! ---> Zone 11
call getfbr('15',nlelt,lstelt)
!==========

do ilelt = 1, nlelt

  ifac = lstelt(ilelt)

  izone        = 11
  ifrlag(ifac) = izone

enddo

! ---> Zone 12
call getfbr('19 or 11',nlelt,lstelt)
!==========

do ilelt = 1, nlelt

  ifac = lstelt(ilelt)

  izone        = 12
  ifrlag(ifac) = izone

enddo

if (irangp.ge.0) then
   do ii = 1, 9
      call parsom(carpart(1,ii))
      call parsom(carpart(2,ii))
   enddo
endif

!===============================================================================
! 4. Injection per particle class into the calculation domain
!===============================================================================

! To provide information about the particle classes,
! we follow a two-step procedure:

!   1) first, the number of particle classes is prescribed
!      for each boundary zone: iusncl (by default, this parameter is set to zero)

!   2) afterwards, for each zone and for each class, we prescribe
!      the physical properties of the particles

! Number of particle classes entering the domain
!   We assign here the number of classes for each zone previously identified.
!
!   This number is zero by default.
!   The maximal number of classes is nclagm (defined in lagpar.h)

! ---> First zone numbered izone = 1: 1 class injected
do i = 1, 9
  izone     = i
  nbclas    = 1
  iusncl(izone) = nbclas
enddo

! For every class associated with a zone,
! we provide the following information.

! iusncl number of classes per zone
! iusclb boundary conditions for the particles
! = ientrl -> zone of particle inlet
! = isortl -> particle outlet
! = irebol -> rebound of the particles
! = idepo1 -> definitive deposition
! = idepo2 -> definitive deposition, but the particle remains in memory
!             (useful only if iensi2 = 1)
! = idepfa -> deposition of the particle with DLVO forces
! = iencrl -> fouling (coal only iphyla = 2)
! = isymtl -> symmetry condition for the particles (zero flux)
! --> For every class associated with a zone,
!     we give the following information.

! iczpar array:
! ============
!    ijnbp : number of particles per class and per zone
!    ijfre : injection frequency. If ijfre = 0, then the injection
!            occurs only at the first absolute iteration.
!    iclst : number of the group to which the particle belongs
!            (only if one wishes to calculate statistics per group)
!    ijuvw : type of condition on the velocity
!              = -1 imposed flow velocity
!              =  0 imposed velocity along the normal direction of the
!                   boundary face, with norm equal to rczpar(iuno)
!              =  1 imposed velocity: we prescribe   rczpar(iupt)
!                                                    rczpar(ivpt)
!                                                    rczpar(iwpt)
!              =  2 user-defined profile
!    ijprpd : type of statistical weight
!              =  1 automatic: we prescribe          rczpar(idebt)
!                                                    rczpar(ipoit)
!              =  2 user-defined profile
!    ijprtp : type of temperature condition
!              =  1 imposed temperature: we prescribe rczpar(itpt)
!              =  2 user-defined profile
!    ijprdp : type of diameter condition
!              =  1 imposed diameter: we prescribe  rczpar(idpt)
!                                                   rczpar(ivdpt)
!              =  2 user-defined profile
!    ijprtp : type of temperature condition
!              =  1 imposed temperature: we prescribe rczpar(itpt)
!              =  2 user-defined profile
!    inuchl : number of the coal of the particle (only if iphyla = 2)
!    irawcl : type of coal injection composition (only if iphyla = 2)
!              = 0 coal injected with an user-defined composition
!              = 1 raw coal injection

! rczpar array:
! ============
!    iuno  : Norm of the velocity (m/s)
!    iupt  : Velocity along the X axis, for each class and for each zone (m/s)
!    ivpt  : Velocity along the Y axis, for each class and for each zone (m/s)
!    iwpt  : Velocity along the Z axis, for each class and for each zone (m/s
!    ipoit : Statistical weight (number of samples) associated
!            to the particle (automatically computed to respect a mass
!            flow rate if it is defined)
!    idebt : Mass flow rate (kg/s)

!    Physical characteristics:
!      idpt   : diameter (m)
!      ivdpt  : standard deviation of the diameter (m)
!      iropt  : density (kg/m3)
!      itpt   : temperature in Celsius degress (no enthalpy)
!      icpt   : specific heat (J/kg/K)
!      iepsi  : emissivity (if =0 then no radiative effect is taken into account)

do i = 1,9
  izone     = i
  nbclas    = iusncl(izone)
  iusclb (izone)         =  ientrl

  n1 = 1
  call normalen(n1,rd)
  dp = 49.3d-6 + rd(1) * 4.85d-6
  if ( dp.lt.30.d-6 ) dp = 30.d-6
  if ( dp.gt.70.d-6 ) dp = 70.d-6

  np = 0
  call inletp(np,carpart(1,i),carpart(2,i),poid,dtp,dp)

  do iclas  = 1, nbclas

    ! Ensure defaults are set

    call lagr_init_zone_class_param(iczpar, rczpar)
    !==============================

    ! Now define parameters for this class and zone

    iczpar(ijnbp) = np
    iczpar(ijfre) = 1

    if (nbclst.gt.0) then
      iczpar(iclst) = 1
    endif

    iczpar(ijuvw) = 1
    rczpar(iupt)  = 1.1d0
    rczpar(ivpt)  = 0.0d0
    rczpar(iwpt)  = 0.0d0
    iczpar(ijprpd)= 1
    rczpar(ipoit) = poid
    rczpar(idebt) = 0.d0

    ! if the physics is " simple"

    if (iphyla.eq.0) then

      ! Mean value and standard deviation of the diameter

      iczpar(ijprdp) = 1
      rczpar(idpt)  = dp
      rczpar(ivdpt) = 0

      ! Density

      rczpar(iropt) = 2590.d0

    endif

    ! Complete definition of parameters for this class and zone

    call lagr_define_zone_class_param(iclas, izone, iczpar, rczpar)
    !================================

  enddo

enddo

! ---> Second zone, numbered izone = 2
!        IUSCLB : rebound of the particle

izone     = 10
iusclb (izone)         =  isymtl

izone     = 11
iusclb (izone)         =  irebol

izone     = 12
iusclb (izone)         =  isortl


!===============================================================================

!--------
! Formats
!--------

!----
! End
!----

! Deallocate the temporary array
deallocate(lstelt)

return
end subroutine uslag2
